function [lambda, T, event] = Gillespie(psi)
% Gillespie.m applies the Gillespie algorithm to a stochastic system.
% Waiting times to the next event are drawn from an exponential
% distribution, with a rate parameter equal to the sum of the probabilities
% per unit time of all events. The next event is chosen at random, with a
% probability that is proportional to the probability per unit time of the
% event.
%  
% INPUTS: psi = psi(t), a 1xm array storing the no. of movement events per unit time for m individuals
% OUTPUTS: lambda = rate parameter for combined Poisson process
%          T = tau, waiting time to next event
%          event = the individual that will undergo the next event

        lambda=sum(psi); % rate parameter = sum(psi's) for that time step 
        mu=1/lambda; % mean parameter = 1/(rate parameter)
        T=exprnd(mu); % draw a random time to next event (i.e. waiting time) from an exponential dist
       
        % Select cell to move/wait at next event
        csum=cumsum(psi); % cumulative sum of psi's at that time step (i.e. all psi's lying along a line between [0,lambda])
        R=lambda*rand; % draw random number from uniform dist on interval [0,lambda]
        event=find(csum>R, 1, 'first'); % find first indice of csum that is > R. Cell "event" selected for next event
        
end

