function [W,GradV_x,GradV_y,psi,neg_mot,lambda,T,event,X,Y,mag,t] = LF_IBM_2D(m,n,rmu,mi,a,beta,var1,var2,ICx,ICy,Lx,Ly)

% Lattice-free individual-based model (IBM) of collective cell movement in two-dimensional space, implementing Gillespie Algorithm: 
%
% Stochastic system involving m Poisson processes 
% Time is continuous and evolves via discrete time jumps T from one movement
% event to the next.
% Cell locations defined by cartesian coordinates.
% Cells have a motility rate comprising an intrinsic and neighbour-dep
% component. Direction of movement is neighbour-dependent.
% Finite (rectangular) domain centred on (0,0): periodic boundaries at x=xl, x=xr, y=yb and y=yt 
%
% INPUTS:     m = number of cells
%             n = number of events
%           rmu = mean step length parameter = 1/(rate parameter), for random generation
%                 of step length from a normal distribution
%            mi = intrinsic motility rate
%             a = interaction strength parameter (height of Gaussian peak)
%          beta = interaction strength parameter for directional bias (move_vonMises.m)
%          var1 = variance of interaction kernel w(x,y) (n.dep motility rate)
%          var2 = variance of interaction kernel v(x,y) (n.dep direction)
%      ICx, ICy = initial x and y coordinates for m cells are random 
%                 variables drawn from a uniform distribution on [-ICx,ICx]
%                 and [-ICy,ICy], respectively.
%        Lx, Ly = domain width and height, respectively 
%         
% OUTPUTS:    W = height of crowding surface W(x,y) at each cell's location
%       GradV_x = x component of gradient of crowding surface V(x,y) at each cell's location 
%       GradV_y = y component of gradient of crowding surface V(x,y) at each cell's location 
%           psi = no. movement events per unit time for each cell at each event
%       neg_mot = negative motility rate count
%        lambda = lambda(t) rate parameter for combined Poisson process, i.e. in Gillespie algorithm, for each event
%             T = set of discrete time jumps tau, i.e. waiting times to next event 
%         event = the individual that will undergo the next event
%           X,Y = locations of m cells over n events
%           mag = magnitude of grad(G) at location of every cell selected to move 
%             t = time (cumulative)
% USAGE: e.g. [W,GradV_x,GradV_y,psi,neg_mot,lambda,T,event,X,Y,mag,t] = LF_IBM_2D(200,20000,0.2,10,-1,0.5,0.25,0.25,-10,10,20,20);
%

% Define parameters and preallocate matrices:

X=zeros(n,m); % preallocate location matrix X
Y=zeros(n,m); 
psi=zeros(n-1,m); 
neg_mot=0; % initialise negative motility rate counter
lambda=zeros(1,n-1); 
T=zeros(1,n-1); 
event=zeros(1,n-1); 
t=zeros(1,n); 
% Gaussian function variables:
% a=1/(2*pi*var); % peak height for normalised interaction kernel in 2D
w=zeros(m,m);
W=zeros(n-1,m); 
gradv_x=zeros(m,m); 
gradv_y=zeros(m,m);
GradV_x=zeros(n-1,m); 
GradV_y=zeros(n-1,m); 
mag=zeros(1,n-1); % magnitude of grad(V) at location of every cell selected to move 

% Initial conditions:
X(1,:)=-ICx + 2*ICx*rand(1,m); % cell x coordinates at t=0, Poisson distribution IC
Y(1,:)=-ICy + 2*ICy*rand(1,m); % cell y coordinates at t=0

% periodic boundaries at x=xl, x=xr, y=yb and y=yt :
xr=Lx/2;
xl=-xr;
yt=Ly/2;
yb=-yt;

% Initialise interaction weights:
for i0=1:m % for every cell
        
        % Weight the interaction between cell i0 and each of its neighbours. 
        % Find the gradient of the interaction (i.e. direction and magnitude).     
        [w(i0,:),~,~] = interact_2D(X(1,i0), Y(1,i0), X(1,:), Y(1,:), var1, a, xl, xr, yb, yt, Lx, Ly);  % weighted by interaction kernel for motility w
        [~,gradv_x(i0,:), gradv_y(i0,:)] = interact_2D(X(1,i0), Y(1,i0), X(1,:), Y(1,:), var2, beta, xl, xr, yb, yt, Lx, Ly); % gradient of interaction kernel for direction v 
           
end


% Simulate cell movement:
for i=1:n-1 % for every discrete time jump (every event)  
   
        % Find total weight of interaction with neighbourhood W(x,y) for each cell
        W(i,:)=sum(w,2)-a; % and minus effect of individual on itself
        
        % Calculate gradient of total interaction with neighbourhood V(x,y) for each cell.
        GradV_x(i,:)=sum(gradv_x,2); % x component of gradient of V(x,y) (i.e. grad(V)) for each cell
        GradV_y(i,:)=sum(gradv_y,2); % y component of gradient of V(x,y) (i.e. grad(V)) for each cell
        
        % Calculate the number of movement events per unit time (psi) for each cell for the next event
        mot=mi + W(i,:); % intrinsic + neighbour-dependent motility rate
        psi(i,:)=max(0, mot); % motility rate >=0
        if any(mot<0)
            neg_mot=neg_mot + length(find(mot<0)); % update negative motility rate counter
        end
       
        % Apply Gillespie algorithm to determine waiting time T to next movement
        % event and to select a cell to undergo the next event.
        [lambda(i), T(i), event(i)] = Gillespie(psi(i,:));
               
        % Specify same location for all cells at next time jump i+1.
        X(i+1,:)=X(i,:);  % cell waits, 
        Y(i+1,:)=Y(i,:);  % no movement in x or y direction
        
        % Move selected cell and update system to reflect the new location of that cell.
        [X(i+1,event(i)), Y(i+1,event(i)), mag(i)] = move_vonMises(GradV_x(i,event(i)), GradV_y(i,event(i)), X(i,event(i)), Y(i,event(i)), rmu, xl, xr, yb, yt, Lx, Ly);  
       
        % Update interaction weights to reflect cell selected cell's movement
        [w(event(i),:),~,~] = interact_2D(X(i+1,event(i)), Y(i+1,event(i)), X(i+1,:), Y(i+1,:), var1, a, xl, xr, yb, yt, Lx, Ly);
        [~,gradv_x(event(i),:),gradv_y(event(i),:)] = interact_2D(X(i+1,event(i)), Y(i+1,event(i)), X(i+1,:), Y(i+1,:), var2, beta, xl, xr, yb, yt, Lx, Ly);
        w(:,event(i)) = w(event(i),:); % Make use of w's symmetry
        gradv_x(:,event(i)) = -gradv_x(event(i),:); % N.B. gradients of an interacting pair have same magnitude but opposite sign 
        gradv_y(:,event(i)) = -gradv_y(event(i),:); 
        
        % update cumulative time t by waiting time T(i).
        t(i+1)=t(i)+T(i);       
end

    % Warnings: 
    if neg_mot>0 % issue warning if motility rate is ever negative
        disp('Warning: negative motility rate');
    end
    
end

