function [g, gradx, grady] = interact_2D(x_i, y_i, x_j, y_j, var, a, xl, xr, yb, yt, Lx, Ly)
% interact_2D.m weights the interaction between an individual i at
% (x_i,y_i) with a neighbour j at (x_j,y_j) using an interaction kernel g (a 2D Gaussian function). 
% The interaction kernel g is expressed in terms of the displacement xi=(xi_x,xi_y) 
% from the individual at (x_i,y_i) to the neighbour at (x_j,y_j).
% Also calculates the gradient with respect to xi, i.e. dg/d(xi) of the 
% interaction (i.e. a vector with direction and magnitude). 
% Implements periodic BCs.
%   
% INPUTS: x_i,y_i = x and y coordinates for an individual i
%        x_j, y_j = (1,m) arrays storing neighbour locations (x and y coordinates)
%             var = variance of Gaussian g(xi), i.e. spread
%               a = interaction strength constant (height of gaussian peak)
%     xl,xr,yb,yt = periodic boundaries at x=xl, x=xr, y=yb and y=yt 
%          Lx, Ly = domain width and height, respectively
% OUTPUTS: g(xi) = 1xm array storing interactions between i and each j weighted by 2D circular Gaussian function (i.e. w(xi) or v(xi))
%          gradx = x component of grad(g), i.e. partial derivative of g with respect to xi_x  
%          grady = y component of grad(g), i.e. partial derivative of g with respect to xi_y 

% Calculate displacement xi from individual i to neighbour j.
xi_x = x_j - x_i; % Component of xi in x direction
xi_y = y_j - y_i; % Component of xi in y direction

xi_x = xi_x + (xi_x<xl)*Lx - (xi_x>xr)*Lx; % implement periodic boundaries
xi_y = xi_y + (xi_y<yb)*Ly - (xi_y>yt)*Ly;

g=a*exp(-(xi_x.^2+xi_y.^2)./(2*var)); % weight of interaction g(xi)

gradx=g.*(-xi_x./var); % x component of gradient of g(xi) (i.e. grad(g))
grady=g.*(-xi_y./var); % y component of gradient of g(xi) (i.e. grad(g))

end

