function [x_new, y_new, mag] = move_vonMises(Gradx, Grady, x_old, y_old, rmu, xl, xr, yb, yt, Lx, Ly)
% move_vonMises.m moves cell in a random direction dir, drawn from a von
% Mises distribution with mean = the direction of, and concentration k = magnitude of the directional vector 
% grad(G) at the selected cell's location. Bounded domain in xl<x<xr,
% yb<y<yt with periodic boundary conditions.
%   
% INPUTS:         Gradx, Grady = x and y components of grad(G) at selected
%                                cell's location
%                 x_old, y_old = selected cell's location at time of last event
%                          rmu = mean parameter = 1/(rate parameter), for random generation
%                                of step length from a normal distribution for selected cell
%               xl, xr, yb, yt = periodic boundaries at x=xl, x=xr, y=yb and y=yt 
%                       Lx, Ly = domain width and height, respectively
% OUTPUTS:        x_new, y_new = selected cell's location after movement
% USAGE: E.g. Decide direction of movement and step length for cell "event(i)": 
% [X(i+1,event(i)), Y(i+1,event(i)), mag(i)] = move_vonMises(GradV_x(i,event(i)), GradV_y(i,event(i)), X(i,event(i)), Y(i,event(i)), rmu, xl, xr, yb, yt, Lx, Ly);  
       
        mag=sqrt(Gradx^2 + Grady^2); % magnitude of grad(G) for selected cell "event(i)"
        
       % Convert x-step and y-step of grad(G) (i.e. direction of steepest
       % slope) to an angle theta0 for cell "event(i)" (where theta0=0 is East)
        theta0 = cart2pol(Gradx, Grady);

        % Determine direction of movement for cell "event(i)":
        dir=circ_vmrnd(theta0,mag,1); % random angle in [0,2*pi] drawn from a von Mises distribution with mean=theta and concentration parameter k=mag
        
       % Step length r for cell "event(i)" drawn from normal distribution
       % with mean rmu and standard deviation rmu/4
        r=normrnd(rmu,rmu/4); 
            while r<0 % if step length r is negative discard and draw a new r
                r=normrnd(rmu,rmu/4); 
            end
        
       % Proposed movement for x and y direction. Cell "event(i)" attempts to move a distance r in direction dir.
        xstep = x_old + r*cos(dir); 
        ystep = y_old + r*sin(dir);
         
       % Periodic BCs: if the movement step doesn't take cell outside of boundary then step complete. 
       % If step takes cell beyond a boundary then it reappears at the opposite boundary and completes 
       % its movement.       
        x_new = xstep + (xstep<xl)*Lx - (xstep>xr)*Lx;
        y_new = ystep + (ystep<yb)*Ly - (ystep>yt)*Ly;
       
       % New location stored at time step i+1
end

