function [dist,F,d] = pairwise_dist_2D(x,y,xl,xr,yb,yt)
% Pairwise_dist_2D.m calculates distances between all pairs of cells.
% Implements periodic boundaries.
%
% INPUTS:        x, y - x- and y-coordinates of all individuals at a particular time
%         xl,xr,yb,yt - domain boundaries x=xl,x=xr,y=yb,y=yt
% OUTPUTS:       F, d - the empirical cumulative distribution function (cdf) F, 
%                   evaluated at the points in d, using the data in the vector 
%                   pairwise (data sorted in ascending order).
%                dist - {m(m-1)}x1 array of pairwise distances 

m=length(x);
Lx=xr-xl;
Ly=yt-yb;
D=zeros(m);
xi_x=zeros(m);
xi_y=zeros(m);

for k=1:m % for each cell
    xi_x(k,:)= x-x(k); % transform locations such that domain centred on cell k,
    xi_y(k,:)= y-y(k); % i.e. calculate displacements between cells
    
    xi_x(k,:)=xi_x(k,:) + (xi_x(k,:)<xl)*Lx - (xi_x(k,:)>xr)*Lx; % implement periodic boundary conditions
    xi_y(k,:)=xi_y(k,:) + (xi_y(k,:)<yb)*Ly - (xi_y(k,:)>yt)*Ly;
        
    D(k,:)=sqrt((xi_x(k,:)).^2 + (xi_y(k,:)).^2); % distance between cell k and each of its neighbours
    D(k,k) = NaN; % make self-pairs NaN
end

dist=D(isfinite(D(:))); % column vector of distances between cell pairs

% Calculate the empirical cumulative distribution function (cdf), 
% evaluated at the points in d, using the data in the vector dist 
[F,d] = ecdf(dist);

end

